@@ -159,6 +159,13 @@ fluid_audio_driver_t *new_fluid_sndmgr_audio_driver2(fluid_settings_t *settings,
 void delete_fluid_sndmgr_audio_driver(fluid_audio_driver_t *p);
 #endif
 
+#if KAI_SUPPORT
+fluid_audio_driver_t *new_fluid_kai_audio_driver(fluid_settings_t *settings,
+        fluid_synth_t *synth);
+void delete_fluid_kai_audio_driver(fluid_audio_driver_t *p);
+void fluid_kai_audio_driver_settings(fluid_settings_t *settings);
+#endif
+
 #if DART_SUPPORT
 fluid_audio_driver_t *new_fluid_dart_audio_driver(fluid_settings_t *settings,
         fluid_synth_t *synth);
