@@ -331,7 +331,7 @@ fluid_rvoice_write(fluid_rvoice_t *voice, fluid_real_t *dsp_buf)
 
     /******************* vol env **********************/
 
-    fluid_adsr_env_calc(&voice->envlfo.volenv, 1);
+    fluid_adsr_env_calc(&voice->envlfo.volenv);
     fluid_check_fpe("voice_write vol env");
 
     if(fluid_adsr_env_get_section(&voice->envlfo.volenv) == FLUID_VOICE_ENVFINISHED)
@@ -341,7 +341,7 @@ fluid_rvoice_write(fluid_rvoice_t *voice, fluid_real_t *dsp_buf)
 
     /******************* mod env **********************/
 
-    fluid_adsr_env_calc(&voice->envlfo.modenv, 0);
+    fluid_adsr_env_calc(&voice->envlfo.modenv);
     fluid_check_fpe("voice_write mod env");
 
     /******************* lfo **********************/
@@ -354,11 +354,13 @@ fluid_rvoice_write(fluid_rvoice_t *voice, fluid_real_t *dsp_buf)
     /******************* amplitude **********************/
 
     count = fluid_rvoice_calc_amp(voice);
-
-    if(count <= 0)
+    if(count == 0)
     {
-        return count; /* return -1 if voice is quiet, 0 if voice has finished */
+        // Voice has finished, remove from dsp loop
+        return 0;
     }
+    // else if count is negative, still process the voice
+
 
     /******************* phase **********************/
 
@@ -420,34 +422,69 @@ fluid_rvoice_write(fluid_rvoice_t *voice, fluid_real_t *dsp_buf)
         voice->dsp.phase_incr = 1;
     }
 
+    /* loop mode release? if not in release, the voice is silent
+     * note: this intentionally processes the volenv before returning silence,
+     * since that's what polyphone does (PR #1400) */
+    if(voice->dsp.samplemode == FLUID_START_ON_RELEASE && fluid_adsr_env_get_section(&voice->envlfo.volenv) < FLUID_VOICE_ENVRELEASE)
+    {
+        return -1;
+    }
+
     /* voice is currently looping? */
     is_looping = voice->dsp.samplemode == FLUID_LOOP_DURING_RELEASE
                  || (voice->dsp.samplemode == FLUID_LOOP_UNTIL_RELEASE
                      && fluid_adsr_env_get_section(&voice->envlfo.volenv) < FLUID_VOICE_ENVRELEASE);
 
+    /*************** resonant filter ******************/
+    // Only "prepare" the filter here, the filter itself will be applied in the dsp_interpolation routines below.
+    // This is to satisfy SF2 Section 9.1.8, particularly, the filtered output must be gain-adjusted by the volEnv.
+    // Applying the filter after applying the gain from the volEnv might cause audible clicks for when turning off
+    // voices that are filtered by a high Q, see https://github.com/FluidSynth/fluidsynth/issues/1427
+    //
+    // Note that at this point we are using voice->dsp.output_rate which is set to the synth's output rate, because
+    // the filter will receive the interpolated waveform.
+
+    fluid_iir_filter_calc(&voice->resonant_filter, voice->dsp.output_rate,
+                          fluid_lfo_get_val(&voice->envlfo.modlfo) * voice->envlfo.modlfo_to_fc +
+                          modenv_val * voice->envlfo.modenv_to_fc);
+
+    /* additional custom filter - only uses the fixed modulator, no lfos... */
+    fluid_iir_filter_calc(&voice->resonant_custom_filter, voice->dsp.output_rate, 0);
+
     /*********************** run the dsp chain ************************
      * The sample is mixed with the output buffer.
      * The buffer has to be filled from 0 to FLUID_BUFSIZE-1.
      * Depending on the position in the loop and the loop size, this
      * may require several runs. */
 
+    if(count < 0)
+    {
+        // The voice is quite, i.e. either in delay phase or zero volume.
+        // We need to update the rvoice's dsp phase, as the delay phase shall not "postpone" the sound, rather
+        // it should be played silently, see https://github.com/FluidSynth/fluidsynth/issues/1312
+        //
+        // Currently, this does access the sample buffers, which is redundant and could be optimized away.
+        // On the other hand, entering this if-clause is not supposed to happen often.
+        return fluid_rvoice_dsp_interpolate_none(voice, dsp_buf, is_looping);
+    }
+
     switch(voice->dsp.interp_method)
     {
     case FLUID_INTERP_NONE:
-        count = fluid_rvoice_dsp_interpolate_none(&voice->dsp, dsp_buf, is_looping);
+        count = fluid_rvoice_dsp_interpolate_none(voice, dsp_buf, is_looping);
         break;
 
     case FLUID_INTERP_LINEAR:
-        count = fluid_rvoice_dsp_interpolate_linear(&voice->dsp, dsp_buf, is_looping);
+        count = fluid_rvoice_dsp_interpolate_linear(voice, dsp_buf, is_looping);
         break;
 
     case FLUID_INTERP_4THORDER:
     default:
-        count = fluid_rvoice_dsp_interpolate_4th_order(&voice->dsp, dsp_buf, is_looping);
+        count = fluid_rvoice_dsp_interpolate_4th_order(voice, dsp_buf, is_looping);
         break;
 
     case FLUID_INTERP_7THORDER:
-        count = fluid_rvoice_dsp_interpolate_7th_order(&voice->dsp, dsp_buf, is_looping);
+        count = fluid_rvoice_dsp_interpolate_7th_order(voice, dsp_buf, is_looping);
         break;
     }
 
@@ -455,21 +492,10 @@ fluid_rvoice_write(fluid_rvoice_t *voice, fluid_real_t *dsp_buf)
 
     if(count == 0)
     {
+        // voice has finished
         return count;
     }
 
-    /*************** resonant filter ******************/
-
-    fluid_iir_filter_calc(&voice->resonant_filter, voice->dsp.output_rate,
-                          fluid_lfo_get_val(&voice->envlfo.modlfo) * voice->envlfo.modlfo_to_fc +
-                          modenv_val * voice->envlfo.modenv_to_fc);
-
-    fluid_iir_filter_apply(&voice->resonant_filter, dsp_buf, count);
-
-    /* additional custom filter - only uses the fixed modulator, no lfos... */
-    fluid_iir_filter_calc(&voice->resonant_custom_filter, voice->dsp.output_rate, 0);
-    fluid_iir_filter_apply(&voice->resonant_custom_filter, dsp_buf, count);
-
     return count;
 }
 
@@ -493,7 +519,8 @@ fluid_rvoice_buffers_check_bufnum(fluid_rvoice_buffers_t *buffers, unsigned int
 
     for(i = buffers->count; i <= bufnum; i++)
     {
-        buffers->bufs[i].amp = 0.0f;
+        buffers->bufs[i].target_amp = 0.0f;
+        buffers->bufs[i].current_amp = 0.0f;
     }
 
     buffers->count = bufnum + 1;
@@ -512,7 +539,7 @@ DECLARE_FLUID_RVOICE_FUNCTION(fluid_rvoice_buffers_set_amp)
         return;
     }
 
-    buffers->bufs[bufnum].amp = value;
+    buffers->bufs[bufnum].target_amp = value;
 }
 
 DECLARE_FLUID_RVOICE_FUNCTION(fluid_rvoice_buffers_set_mapping)
